"""Functions that handle files in the context of Scout uploading."""

import logging
import re
from pathlib import Path
from typing import Any

from housekeeper.store.models import File, Version

from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.lims import LimsAPI
from cg.constants import Priority
from cg.constants.constants import Workflow
from cg.constants.subject import SCOUT_PRIORITIZED_STATUS, RelationshipStatus
from cg.meta.upload.scout.hk_tags import CaseTags, SampleTags
from cg.models.scout.scout_load_config import (
    ScoutIndividual,
    ScoutLoadConfig,
    ScoutMipIndividual,
    ScoutNalloIndividual,
    ScoutRarediseaseIndividual,
)
from cg.store.models import Analysis, Case, CaseSample, Sample

LOG = logging.getLogger(__name__)


class ScoutConfigBuilder:
    """Base class for handling files that should be included in Scout upload."""

    def __init__(self, lims_api: LimsAPI):
        self.lims_api: LimsAPI = lims_api
        self.case_tags: CaseTags
        self.sample_tags: SampleTags

    def add_common_info_to_load_config(
        self, load_config: ScoutLoadConfig, analysis: Analysis
    ) -> None:
        """Add the mandatory common information to a Scout load config object."""
        load_config.analysis_date = analysis.completed_at or analysis.started_at
        load_config.default_gene_panels = analysis.case.panels
        load_config.family = analysis.case.internal_id
        load_config.family_name = analysis.case.name
        load_config.owner = analysis.case.customer.internal_id
        load_config.status = (
            SCOUT_PRIORITIZED_STATUS if analysis.case.priority >= Priority.priority else None
        )
        load_config.synopsis = analysis.case.synopsis
        self.include_cohorts(load_config=load_config, analysis=analysis)
        self.include_phenotype_groups(load_config=load_config, analysis=analysis)
        self.include_phenotype_terms(load_config=load_config, analysis=analysis)

    def run_madeline(self, family_obj: Case) -> Path:
        """Generate a madeline file for an analysis. Use customer sample names."""
        samples = [
            {
                "sample": link_obj.sample.name,
                "sex": link_obj.sample.sex,
                "father": link_obj.father.name if link_obj.father else None,
                "mother": link_obj.mother.name if link_obj.mother else None,
                "status": link_obj.status,
            }
            for link_obj in family_obj.links
        ]
        svg_path: Path = self.madeline_api.run(family_id=family_obj.name, samples=samples)
        return svg_path

    @staticmethod
    def is_multi_sample_case(load_config: ScoutLoadConfig) -> bool:
        return len(load_config.samples) > 1

    @staticmethod
    def is_family_case(load_config: ScoutLoadConfig) -> bool:
        """Check if there are any linked individuals in a case."""
        for sample in load_config.samples:
            if sample.mother and sample.mother != "0":
                return True

            if sample.father and sample.father != "0":
                return True
        return False

    def include_pedigree_picture(self, load_config: ScoutLoadConfig, analysis: Analysis) -> None:
        """Run Madeline only for cases with multiple related samples."""
        if self.is_multi_sample_case(load_config=load_config):
            if self.is_family_case(load_config=load_config):
                svg_path: Path = self.run_madeline(analysis.case)
                load_config.madeline = str(svg_path)
            else:
                LOG.info("family of unconnected samples - skip pedigree graph")
        else:
            LOG.info("family of 1 sample - skip pedigree graph")

    @staticmethod
    def remove_chromosome_substring(file_path: str | None) -> str | None:
        """Remove a file's suffix and identifying integer or X/Y
        Example:
        `/some/path/gatkcomb_rhocall_vt_af_chromograph_sites_X.png` becomes
        `/some/path/gatkcomb_rhocall_vt_af_chromograph_sites_`"""
        if file_path is None:
            return file_path
        return re.split(r"(?:[1-9]|1[0-9]|2[0-2]|[XY])\.png$", file_path)[0]

    def get_sample_information(
        self, load_config: ScoutLoadConfig, analysis: Analysis, hk_version: Version
    ) -> None:
        LOG.info("Building samples")
        db_sample: CaseSample
        for db_sample in analysis.case.links:
            load_config.samples.append(
                self.build_config_sample(case_sample=db_sample, hk_version=hk_version)
            )

    def build_config_sample(self, case_sample: CaseSample, hk_version: Version) -> ScoutIndividual:
        """Build a sample with rnafusion specific information."""
        workflow = case_sample.case.data_analysis
        if workflow == Workflow.RAREDISEASE:
            config_sample = ScoutRarediseaseIndividual()
        elif workflow == Workflow.MIP_DNA:
            config_sample = ScoutMipIndividual()
        elif workflow == Workflow.NALLO:
            config_sample = ScoutNalloIndividual()
        elif workflow == Workflow.RNAFUSION:
            config_sample = ScoutIndividual()
        self.add_common_sample_info(config_sample=config_sample, case_sample=case_sample)
        self.add_common_sample_files(
            config_sample=config_sample, case_sample=case_sample, hk_version=hk_version
        )
        return config_sample

    def add_common_sample_info(
        self, config_sample: ScoutIndividual, case_sample: CaseSample
    ) -> None:
        """Add the information to a sample that is common for different analysis types."""
        sample_id: str = case_sample.sample.internal_id
        LOG.info(f"Building sample {sample_id}")
        lims_sample: dict[str, Any] = self.lims_api.sample(sample_id)
        config_sample.sample_id = sample_id
        config_sample.sex = case_sample.sample.sex
        config_sample.phenotype = case_sample.status
        config_sample.analysis_type = (
            case_sample.sample.application_version.application.analysis_type
        )
        config_sample.sample_name = case_sample.sample.name
        config_sample.tissue_type = lims_sample.get("source", "unknown")
        config_sample.subject_id = case_sample.sample.subject_id
        config_sample.father = (
            case_sample.father.internal_id
            if case_sample.father
            else RelationshipStatus.HAS_NO_PARENT
        )
        config_sample.mother = (
            case_sample.mother.internal_id
            if case_sample.mother
            else RelationshipStatus.HAS_NO_PARENT
        )

    def add_common_sample_files(
        self,
        config_sample: ScoutIndividual,
        case_sample: CaseSample,
        hk_version: Version,
    ) -> None:
        """Add common sample files for different analysis types."""
        LOG.info(f"Adding common files for sample {case_sample.sample.internal_id}")
        self.include_sample_alignment_file(config_sample=config_sample, hk_version=hk_version)
        self.include_sample_files(config_sample=config_sample, hk_version=hk_version)

    def build_load_config(self, hk_version: Version, analysis: Analysis) -> ScoutLoadConfig:
        """Build a load config for uploading a case to Scout."""
        raise NotImplementedError

    def include_sample_files(self, config_sample: ScoutIndividual, hk_version: Version) -> None:
        """Include all files that are used on sample level in Scout."""
        raise NotImplementedError

    def include_case_files(self, load_config: ScoutLoadConfig, hk_version: Version) -> None:
        """Include all files that are used on case level in Scout."""
        raise NotImplementedError

    def include_phenotype_terms(self, load_config: ScoutLoadConfig, analysis: Analysis) -> None:
        LOG.info("Adding phenotype terms to Scout load config")
        phenotype_terms: set[str] = set()
        link_obj: CaseSample
        for link_obj in analysis.case.links:
            sample_obj: Sample = link_obj.sample
            for phenotype_term in sample_obj.phenotype_terms:
                LOG.debug(
                    f"Adding term {phenotype_term} from sample {sample_obj.internal_id} to "
                    f"phenotype terms",
                )
                phenotype_terms.add(phenotype_term)
        if phenotype_terms:
            load_config.phenotype_terms = list(phenotype_terms)

    def include_phenotype_groups(self, load_config: ScoutLoadConfig, analysis: Analysis) -> None:
        LOG.info("Adding phenotype groups to Scout load config")
        phenotype_groups: set[str] = set()
        link_obj: CaseSample
        for link_obj in analysis.case.links:
            sample_obj: Sample = link_obj.sample
            for phenotype_group in sample_obj.phenotype_groups:
                LOG.debug(
                    f"Adding group {phenotype_group} from sample { sample_obj.internal_id} to "
                    f"phenotype groups",
                )
                phenotype_groups.add(phenotype_group)
        if phenotype_groups:
            load_config.phenotype_groups = list(phenotype_groups)

    def include_cohorts(self, load_config: ScoutLoadConfig, analysis: Analysis) -> None:
        LOG.info("Including cohorts to Scout load config")
        cohorts: list[str] = analysis.case.cohorts
        if cohorts:
            LOG.debug(f"Adding cohorts {', '.join(cohorts)}")
            load_config.cohorts = cohorts

    def include_cnv_report(self, load_config: ScoutLoadConfig, hk_version: Version) -> None:
        LOG.info("Include CNV report to case")
        load_config.cnv_report = self.get_file_from_hk(
            hk_tags=self.case_tags.cnv_report, hk_version=hk_version
        )

    def include_multiqc_report(self, load_config: ScoutLoadConfig, hk_version: Version) -> None:
        LOG.info("Include MultiQC report to case")
        load_config.multiqc = self.get_file_from_hk(
            hk_tags=self.case_tags.multiqc_report, hk_version=hk_version
        )

    def include_sample_alignment_file(
        self, config_sample: ScoutIndividual, hk_version: Analysis
    ) -> None:
        """Include the alignment file for a sample
        Try if cram file is found, if not: load bam file
        """
        sample_id: str = config_sample.sample_id
        config_sample.alignment_path = self.get_sample_file(
            hk_tags=self.sample_tags.alignment_file,
            sample_id=sample_id,
            hk_version=hk_version,
        )

        if not config_sample.alignment_path:
            self.include_sample_alignment_bam(config_sample=config_sample, hk_version=hk_version)

    def include_sample_alignment_bam(
        self, config_sample: ScoutIndividual, hk_version: Version
    ) -> None:
        sample_id: str = config_sample.sample_id
        config_sample.alignment_path = self.get_sample_file(
            hk_tags=self.sample_tags.bam_file, sample_id=sample_id, hk_version=hk_version
        )

    def get_sample_file(self, hk_tags: set[str], sample_id: str, hk_version: Version) -> str | None:
        """Return a file that is specific for an individual from Housekeeper."""
        if hk_tags:  # skip if no tag found
            tags: set = hk_tags.copy()
            tags.add(sample_id)
            return self.get_file_from_hk(hk_tags=tags, hk_version=hk_version)

    def get_file_from_hk(self, hk_tags: set[str], hk_version: Version) -> str | None:
        """Return the Housekeeper file path as a string."""

        LOG.info(f"Get file with tags {hk_tags}")
        if not hk_tags:
            LOG.debug("No tags provided, skipping")
            return None
        hk_file: File | None = HousekeeperAPI.get_file_from_version(
            version=hk_version, tags=hk_tags
        )
        return hk_file.full_path if hk_file else None



from pydantic import BaseModel

from cg.constants import Workflow
from cg.services.analysis_starter.configurator.abstract_model import CaseConfig


class MicrosaltConfigContent(BaseModel):
    CG_ID_project: str | None
    Customer_ID_project: str | None
    CG_ID_sample: str
    Customer_ID_sample: str
    organism: str
    priority: str
    reference: str | None
    Customer_ID: str
    application_tag: str
    date_arrival: str
    date_sequencing: str
    date_libprep: str
    method_libprep: str
    method_sequencing: str
    sequencing_qc_passed: bool


class MicrosaltCaseConfig(CaseConfig):
    binary: str
    conda_binary: str
    config_file: str
    environment: str
    fastq_directory: str
    workflow: Workflow = Workflow.MICROSALT



import logging
from functools import wraps


from cg.services.deliver_files.file_fetcher.exc import NoDeliveryFilesError

LOG = logging.getLogger(__name__)


def handle_no_delivery_files_error(func):
    """
    Excepts NoDeliveryFilesError and returns.
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except NoDeliveryFilesError as error:
            LOG.error(error)
            return

    return wrapper



import logging

from housekeeper.store.models import Version

from cg.apps.lims import LimsAPI
from cg.constants.housekeeper_tags import HK_DELIVERY_REPORT_TAG
from cg.constants.scout import (
    BALSAMIC_UMI_CASE_TAGS,
    BALSAMIC_UMI_SAMPLE_TAGS,
    GenomeBuild,
    UploadTrack,
)
from cg.meta.upload.scout.balsamic_config_builder import BalsamicConfigBuilder
from cg.meta.upload.scout.hk_tags import CaseTags, SampleTags
from cg.models.scout.scout_load_config import BalsamicUmiLoadConfig, ScoutCancerIndividual
from cg.store.models import Analysis, CaseSample, Sample

LOG = logging.getLogger(__name__)


class BalsamicUmiConfigBuilder(BalsamicConfigBuilder):
    def __init__(self, lims_api: LimsAPI):
        super().__init__(
            lims_api=lims_api,
        )
        self.case_tags: CaseTags = CaseTags(**BALSAMIC_UMI_CASE_TAGS)
        self.sample_tags: SampleTags = SampleTags(**BALSAMIC_UMI_SAMPLE_TAGS)

    def build_load_config(self, hk_version: Version, analysis: Analysis) -> BalsamicUmiLoadConfig:
        LOG.info("Build load config for balsamic case")
        load_config: BalsamicUmiLoadConfig = BalsamicUmiLoadConfig(
            track=UploadTrack.CANCER.value,
            delivery_report=self.get_file_from_hk(
                hk_tags={HK_DELIVERY_REPORT_TAG}, hk_version=hk_version
            ),
        )
        self.add_common_info_to_load_config(load_config=load_config, analysis=analysis)
        load_config.human_genome_build = GenomeBuild.hg19
        load_config.rank_score_threshold = -100
        self.include_case_files(load_config=load_config, hk_version=hk_version)

        LOG.info("Building samples")
        db_sample: CaseSample

        for db_sample in analysis.case.links:
            load_config.samples.append(
                self.build_config_sample(case_sample=db_sample, hk_version=hk_version)
            )

        return load_config

    def include_sample_files(
        self, config_sample: ScoutCancerIndividual, hk_version: Version
    ) -> None:
        LOG.info("Including BALSAMIC specific sample level files")

    def get_balsamic_analysis_type(self, sample: Sample) -> str:
        """Returns a formatted balsamic analysis type."""
        return "panel-umi"



from pathlib import Path


class PipelineExtension:
    def configure(self, case_id: str, case_path: Path):
        """Intended for pipeline specific configurations. If none is needed, this bare class
        can be used."""
        pass



SBATCH_HEADER_TEMPLATE = """#! /bin/bash {use_login_shell}
#SBATCH --job-name={job_name}
#SBATCH --account={account}
#SBATCH --ntasks={number_tasks}
#SBATCH --mem={memory}G
#SBATCH --error={log_dir}/{job_name}.stderr
#SBATCH --output={log_dir}/{job_name}.stdout
#SBATCH --mail-type=FAIL
#SBATCH --mail-user={email}
#SBATCH --time={hours}:{minutes}:00
#SBATCH --qos={quality_of_service}
{optional_headers}

set -eu -o pipefail

log() {{
    NOW=$(date +"%Y-%m-%dT%H:%M:%S")
    echo "[${{NOW}}] $*" 1>&2;
}}

log "Running on: $(hostname)"
"""

DRAGEN_SBATCH_HEADER_TEMPLATE = """#! /bin/bash
#SBATCH --job-name={job_name}
#SBATCH --partition={partition}
#SBATCH --account={account}
#SBATCH --exclusive
#SBATCH --mem=0 # Allocates all memory on the node
#SBATCH --error={log_dir}/{job_name}.stderr
#SBATCH --output={log_dir}/{job_name}.stdout
#SBATCH --mail-type=FAIL
#SBATCH --mail-user={email}
#SBATCH --time={hours}:{minutes}:00
#SBATCH --qos={quality_of_service}

set -eu -o pipefail

ulimit -n 65535
ulimit -u 16384

export PATH=$PATH:/opt/edico/bin

log() {{
    NOW=$(date +"%Y-%m-%dT%H:%M:%S")
    echo "[${{NOW}}] $*" 1>&2;
}}

log "Running on: $(hostname)"
"""

# Double {{ to escape this character
SBATCH_BODY_TEMPLATE = """
error() {{
    {error_body}
    exit 1
}}

trap error ERR

{commands}

"""



from datetime import date, datetime, timedelta

from cg.clients.freshdesk.constants import Status
from cg.constants.priority import Priority
from cg.models.orders.constants import OrderType
from cg.services.orders.constants import ORDER_TYPE_WORKFLOW_MAP
from cg.services.orders.validation.models.existing_sample import ExistingSample
from cg.services.orders.validation.models.order import Order
from cg.services.orders.validation.models.order_with_cases import OrderWithCases
from cg.services.orders.validation.models.order_with_samples import OrderWithSamples
from cg.services.orders.validation.models.sample import Sample as ValidationSample
from cg.store.models import Application, Sample
from cg.store.store import Store

DUE_TIME_BY_PRIORITY: dict[Priority, timedelta] = {
    Priority.express: timedelta(days=7),
    Priority.priority: timedelta(days=14),
    Priority.standard: timedelta(days=21),
    Priority.clinical_trials: timedelta(days=21),
    Priority.research: timedelta(days=60),
}


def contains_existing_data(order: OrderWithCases) -> bool:
    """Check if the order contains any existing data"""
    return any(not case.is_new or case.enumerated_existing_samples for case in order.cases)


def contains_external_data(order: Order, status_db: Store) -> bool:
    """Check if any existing or new sample from the given order is external."""
    existing_samples: list[Sample] = get_existing_samples(order=order, status_db=status_db)
    new_samples: list[ValidationSample] = get_new_samples(order=order)

    if any([sample.is_external for sample in existing_samples]):
        return True

    for sample in new_samples:
        application: Application | None = status_db.get_application_by_tag(sample.application)
        if application and application.is_external:
            return True

    return False


def get_ticket_tags(order: Order, order_type: OrderType, status_db: Store) -> list[str]:
    """Generate ticket tags based on the order and order type"""

    tags: list[str] = [ORDER_TYPE_WORKFLOW_MAP[order_type]]

    if isinstance(order, OrderWithCases):
        if contains_existing_data(order):
            tags.append("existing-data")

    if contains_external_data(order=order, status_db=status_db):
        tags.append("external-data")

    return tags


def contains_only_existing_samples(order: OrderWithCases) -> bool:
    """Check if the order contains only existing samples"""

    if order.enumerated_new_samples:
        return False
    return True


def get_ticket_status(order: Order) -> Status:
    """Get the ticket status based on the order"""

    if isinstance(order, OrderWithCases):
        if contains_only_existing_samples(order=order):
            return Status.OPEN
    return Status.PENDING


def get_due_by_date(priority: Priority) -> date:
    """Get the ticket due by date based on the order priority."""
    due_by: datetime = datetime.now() + DUE_TIME_BY_PRIORITY[priority]
    return due_by.date()


def get_existing_samples(order: Order, status_db: Store) -> list[Sample]:
    existing_samples: list[Sample] = []

    if isinstance(order, OrderWithCases):
        existing_samples.extend(
            [
                sample
                for (_, case) in order.enumerated_existing_cases
                for sample in status_db.get_samples_by_case_id(case.internal_id)
            ]
        )

        existing_samples.extend(
            [
                sample
                for (_, case) in order.enumerated_new_cases
                for (_, existing_sample) in case.enumerated_existing_samples
                if (sample := status_db.get_sample_by_internal_id(existing_sample.internal_id))
            ]
        )

    return existing_samples


def get_new_samples(order: Order) -> list[ValidationSample]:
    new_samples: list[ValidationSample] = []

    if isinstance(order, OrderWithCases):
        new_samples.extend([sample for (_, _, sample) in order.enumerated_new_samples])
    elif isinstance(order, OrderWithSamples):
        new_samples.extend(order.samples)

    return new_samples



from datetime import datetime
from pathlib import Path
from typing import Annotated

from pydantic import BaseModel, BeforeValidator

from cg.services.illumina.backup.validators import (
    validated_pdc_encryption_key_path,
    validated_pdc_sequencing_file_path,
)


class PdcEncryptionKey(BaseModel):
    """Model representing the response from a PDC query."""

    date_time: datetime
    path: Annotated[Path, BeforeValidator(validated_pdc_encryption_key_path)]


class PdcSequencingFile(BaseModel):
    """Model representing the response from a PDC query."""

    date_time: datetime
    path: Annotated[Path, BeforeValidator(validated_pdc_sequencing_file_path)]



"""CLI support to start microsalt"""

import logging
from pathlib import Path
from typing import Any, cast

import rich_click as click

from cg.cli.utils import CLICK_CONTEXT_SETTINGS
from cg.cli.workflow.commands import ARGUMENT_CASE_ID, resolve_compression, store, store_available
from cg.constants import EXIT_FAIL, EXIT_SUCCESS
from cg.constants.cli_options import DRY_RUN
from cg.constants.constants import FileFormat, Workflow
from cg.exc import AnalysisNotReadyError, CgError
from cg.io.controller import WriteFile, WriteStream
from cg.meta.workflow.analysis import AnalysisAPI
from cg.meta.workflow.microsalt import MicrosaltAnalysisAPI
from cg.models.cg_config import CGConfig
from cg.services.analysis_starter.configurator.implementations.microsalt import (
    MicrosaltConfigurator,
)
from cg.services.analysis_starter.factories.configurator_factory import ConfiguratorFactory
from cg.services.analysis_starter.factories.starter_factory import AnalysisStarterFactory
from cg.store.models import Sample

LOG = logging.getLogger(__name__)

OPTION_SAMPLE = click.option(
    "-s",
    "--sample",
    help="Start workflow by providing a sample_id (By default case_id will be used)",
    is_flag=True,
)
OPTION_TICKET = click.option(
    "-t",
    "--ticket",
    help="Start workflow by providing a ticket_id (By default case_id will be used)",
    is_flag=True,
)
ARGUMENT_UNIQUE_IDENTIFIER = click.argument("unique_id", required=True, type=click.STRING)


@click.group(invoke_without_command=True, context_settings=CLICK_CONTEXT_SETTINGS)
@click.pass_context
def microsalt(context: click.Context) -> None:
    """Microbial workflow"""
    AnalysisAPI.get_help(context)
    context.obj.meta_apis["analysis_api"] = MicrosaltAnalysisAPI(config=context.obj)


microsalt.add_command(store)
microsalt.add_command(store_available)
microsalt.add_command(resolve_compression)


@microsalt.command()
@OPTION_TICKET
@OPTION_SAMPLE
@DRY_RUN
@ARGUMENT_UNIQUE_IDENTIFIER
@click.pass_obj
def link(context: CGConfig, ticket: bool, sample: bool, unique_id: str, dry_run: bool) -> None:
    """Link microbial FASTQ files to dedicated analysis folder for a given case, ticket or sample"""
    if dry_run:
        return
    analysis_api: MicrosaltAnalysisAPI = context.meta_apis["analysis_api"]
    case_id, sample_id = analysis_api.resolve_case_sample_id(
        sample=sample, ticket=ticket, unique_id=unique_id
    )
    analysis_api.link_fastq_files(
        case_id=case_id,
        sample_id=sample_id,
    )


@microsalt.command("config-case")
@DRY_RUN
@OPTION_TICKET
@OPTION_SAMPLE
@ARGUMENT_UNIQUE_IDENTIFIER
@click.pass_obj
def config_case(
    context: CGConfig, dry_run: bool, ticket: bool, sample: bool, unique_id: str
) -> None:
    """Create a config file for a case or a sample analysis in microSALT"""

    analysis_api: MicrosaltAnalysisAPI = context.meta_apis["analysis_api"]
    case_id, sample_id = analysis_api.resolve_case_sample_id(
        sample=sample, ticket=ticket, unique_id=unique_id
    )
    sample_objs: list[Sample] = analysis_api.get_samples(case_id=case_id, sample_id=sample_id)

    if not sample_objs:
        LOG.error("No sample found for that ticket/sample_id")
        raise click.Abort

    parameters: list[dict] = [analysis_api.get_parameters(sample_obj) for sample_obj in sample_objs]
    filename: str = sample_id or case_id
    config_case_path: Path = analysis_api.get_config_path(filename=filename)
    if dry_run:
        click.echo(
            WriteStream.write_stream_from_content(content=parameters, file_format=FileFormat.JSON)
        )
        return
    WriteFile.write_file_from_content(
        content=parameters, file_format=FileFormat.JSON, file_path=config_case_path
    )
    LOG.info(f"Saved config {config_case_path}")


@microsalt.command()
@DRY_RUN
@OPTION_TICKET
@OPTION_SAMPLE
@ARGUMENT_UNIQUE_IDENTIFIER
@click.option(
    "-c",
    "--config-case",
    "config_case_path",
    required=False,
    type=click.Path(exists=True, file_okay=True, dir_okay=False, resolve_path=True),
    help="optionally change the config-case",
)
@click.pass_obj
def run(
    context: CGConfig,
    dry_run: bool,
    config_case_path: click.Path,
    ticket: bool,
    sample: bool,
    unique_id: Any,
) -> None:
    """Start microSALT workflow by providing case, ticket or sample id"""

    analysis_api: MicrosaltAnalysisAPI = context.meta_apis["analysis_api"]
    case_id, sample_id = analysis_api.resolve_case_sample_id(
        sample=sample, ticket=ticket, unique_id=unique_id
    )
    fastq_path: Path = analysis_api.get_case_fastq_path(case_id=case_id)
    if not config_case_path:
        filename = sample_id or case_id
        config_case_path: Path = analysis_api.get_config_path(filename=filename)

    if not sample_id:
        analyse_command = [
            "analyse",
            config_case_path.absolute().as_posix(),
            "--input",
            fastq_path.absolute().as_posix(),
        ]
    else:
        analyse_command = [
            "analyse",
            config_case_path.absolute().as_posix(),
            "--input",
            Path(fastq_path, sample_id).absolute().as_posix(),
        ]

    if sample_id or dry_run:
        analysis_api.process.run_command(parameters=analyse_command, dry_run=dry_run)
        return

    try:
        analysis_api.set_statusdb_action(case_id=case_id, action="running")
        analysis_api.process.run_command(parameters=analyse_command, dry_run=dry_run)
        analysis_api.on_analysis_started(case_id)
    except Exception as error:
        LOG.error(f"Failed to run analysis for case {case_id}: {error}")
        analysis_api.set_statusdb_action(case_id=case_id, action=None)
        raise


@microsalt.command()
@ARGUMENT_UNIQUE_IDENTIFIER
@DRY_RUN
@OPTION_TICKET
@OPTION_SAMPLE
@click.pass_context
def start(
    context: click.Context, ticket: bool, sample: bool, unique_id: str, dry_run: bool
) -> None:
    """Start whole microSALT workflow by providing case, ticket or sample id"""
    LOG.info(f"Starting Microsalt workflow for {unique_id}")
    if not (sample or ticket):
        analysis_api: MicrosaltAnalysisAPI = context.obj.meta_apis["analysis_api"]
        analysis_api.prepare_fastq_files(case_id=unique_id, dry_run=dry_run)
    context.invoke(link, ticket=ticket, sample=sample, unique_id=unique_id, dry_run=dry_run)
    context.invoke(config_case, ticket=ticket, sample=sample, unique_id=unique_id, dry_run=dry_run)
    context.invoke(run, ticket=ticket, sample=sample, unique_id=unique_id, dry_run=dry_run)


@microsalt.command("dev-run")
@click.option(
    "-c",
    "--config",
    "config_file_path",
    required=False,
    type=click.Path(exists=True, file_okay=True, dir_okay=False, resolve_path=True),
    help="optionally change the config file path",
)
@ARGUMENT_CASE_ID
@click.pass_obj
def dev_run(
    context: CGConfig,
    config_file_path: click.Path,
    case_id: str,
) -> None:
    """Runs the microSALT workflow for the provided case. Uses new code. Does not generate
    config files."""

    analysis_starter = AnalysisStarterFactory(context).get_analysis_starter_for_case(case_id)
    analysis_starter.run(case_id=case_id, config_file=config_file_path)


@microsalt.command("dev-start")
@ARGUMENT_CASE_ID
@click.pass_context
def dev_start(context: click.Context, case_id: str) -> None:
    """Generates config file, links fastq files and Starts the microSALT analysis
    for the provided case."""
    LOG.info(f"Starting Microsalt workflow for {case_id}")
    analysis_starter = AnalysisStarterFactory(context.obj).get_analysis_starter_for_case(case_id)
    analysis_starter.start(case_id)


@microsalt.command()
@ARGUMENT_CASE_ID
@click.pass_obj
def dev_config_case(context: CGConfig, case_id: str) -> None:
    """Create a config file for a microSALT case."""
    configurator = cast(
        MicrosaltConfigurator, ConfiguratorFactory(context).get_configurator(Workflow.MICROSALT)
    )
    configurator.configure(case_id=case_id)


@microsalt.command()
@click.pass_obj
def dev_start_available(context: CGConfig) -> None:
    """Starts all available microSALT cases."""
    LOG.info("Starting Microsalt workflow.")
    analysis_starter = AnalysisStarterFactory(context).get_analysis_starter_for_workflow(
        Workflow.MICROSALT
    )
    succeeded: bool = analysis_starter.start_available()
    if not succeeded:
        raise click.Abort


@microsalt.command("start-available")
@DRY_RUN
@click.pass_context
def start_available(context: click.Context, dry_run: bool = False):
    """Start full analysis workflow for all cases ready for analysis"""

    analysis_api: MicrosaltAnalysisAPI = context.obj.meta_apis["analysis_api"]

    exit_code: int = EXIT_SUCCESS
    for case in analysis_api.get_cases_to_analyze():
        try:
            context.invoke(start, unique_id=case.internal_id, dry_run=dry_run)
        except AnalysisNotReadyError as error:
            LOG.error(error)
        except CgError as error:
            LOG.error(error)
            exit_code = EXIT_FAIL
        except Exception as error:
            LOG.error(f"Unspecified error occurred: {error}")
            exit_code = EXIT_FAIL
    if exit_code:
        raise click.Abort


@microsalt.command("qc")
@ARGUMENT_UNIQUE_IDENTIFIER
@click.pass_context
def qc_microsalt(context: click.Context, unique_id: str) -> None:
    """Perform QC on a microsalt case."""
    analysis_api: MicrosaltAnalysisAPI = context.obj.meta_apis["analysis_api"]
    metrics_file_path: Path = analysis_api.get_metrics_file_path(unique_id)
    try:
        LOG.info(f"Performing QC on case {unique_id}")
        analysis_api.quality_checker.quality_control(metrics_file_path)
    except IndexError:
        LOG.error(f"No existing analysis directories found for case {unique_id}.")
